/* 
   This file is a part of Color Selection Dialog library.

   Copyright (C) 1999 ICe Machine
   Copyright (C) 1999 Jacek C. Wojdel <J.C.Wojdel@cs.tudelft.nl>
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*! 
  \file qcolordlg.cc
  \brief Implementation of QColorDlg, QHSSelector and QValueSelector.

  This file must be compiled within your project if you want to use any of the
  above classes. In the simplest case you'll probably be only interesed in the
  QColorDlg class and probably it's static function QColorDlg::getColor(). In
  this way you can have color selection feature with just two additional lines
  in your code; one to include qcolordlg.h file and one to call 
  QColorDlg::getColor() function (see QColorDlg description).
*/
#include "qcolordlg.h"
#include <qlayout.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qimage.h>
#include <qpixmap.h>

/*! 
  This method constructs the dialog with given \e parent and \e name asusually 
  with classes derived from QDialog. Contrary to usual QDialog classes it 
  creates the dialog that is modal.

  The implementation of this method is rather long and boring. It contains a 
  lot of geometry management made with ose of Qt layouts, so that the dialog
  should look OK even if the names of some of the labels are substituted with
  a very long (or very short) strings.

  @param parent a widget's parent as passed to QDialog constructor
  @param name a widget's name as passed to QDialog constructor
*/
QColorDlg::QColorDlg(QWidget* parent, const char * name)
  : QDialog(parent, name, TRUE)
{
  cur_color = white;
  isRgbUpdating = isHsvUpdating = false;

  QVBoxLayout* mainLayout = new QVBoxLayout(this, 5, 5);

  QHBoxLayout* xpmslidersLayout = new QHBoxLayout;
  mainLayout->addLayout(xpmslidersLayout, 0);

  QHSSelector* hsSelector = new QHSSelector(this);
  xpmslidersLayout->addWidget(hsSelector,0);

  QValueSelector* vSelector = new QValueSelector(this);
  xpmslidersLayout->addWidget(vSelector,0);

  QGridLayout* slidersLayout = new QGridLayout(6, 3);
  xpmslidersLayout->addLayout(slidersLayout, 0);
  QLabel* label;
  /* BEGIN RGB */
  /* BEGIN: RGB labels */
  label = new QLabel(QCD_RED_NAME, this); 
  label->setFixedSize(label->sizeHint());
  slidersLayout->addWidget(label, 0, 0, AlignVCenter|AlignRight);
  label = new QLabel(QCD_GREEN_NAME, this); 
  label->setFixedSize(label->sizeHint());
  slidersLayout->addWidget(label, 1, 0, AlignVCenter|AlignRight);
  label = new QLabel(QCD_BLUE_NAME, this); 
  label->setFixedSize(label->sizeHint());
  slidersLayout->addWidget(label, 2, 0, AlignVCenter|AlignRight);
  /* END: RGB labels */
  /* BEGIN RGB sliders */
  rSlider = new QSlider(0, 255, 10, cur_color.red(), QSlider::Horizontal, this);
  rSlider->setFixedSize(rSlider->sizeHint());
  slidersLayout->addWidget(rSlider, 0, 1, AlignCenter);
  gSlider = new QSlider(0, 255, 10, cur_color.green(), QSlider::Horizontal, this);
  gSlider->setFixedSize(gSlider->sizeHint());
  slidersLayout->addWidget(gSlider, 1, 1, AlignCenter);
  bSlider = new QSlider(0, 255, 10, cur_color.blue(), QSlider::Horizontal, this);
  bSlider->setFixedSize(bSlider->sizeHint());
  slidersLayout->addWidget(bSlider, 2, 1, AlignCenter);
  /* END: RGB sliders */
  /* BEGIN RGB spinboxes */  
  QSpinBox* rSpin = new QSpinBox(0, 255, 1, this);
  rSpin->setFixedSize(rSpin->sizeHint());
  slidersLayout->addWidget(rSpin, 0, 2, AlignCenter);
  QSpinBox* gSpin = new QSpinBox(0, 255, 1, this);
  gSpin->setFixedSize(gSpin->sizeHint());
  slidersLayout->addWidget(gSpin, 1, 2, AlignCenter);
  QSpinBox* bSpin = new QSpinBox(0, 255, 1, this);
  bSpin->setFixedSize(bSpin->sizeHint());
  slidersLayout->addWidget(bSpin, 2, 2, AlignCenter);
  /* END: RGB spinboxes */  
  /* END: RGB */

  /* BEGIN: HSV */
  /* BEGIN: HSV labels */
  label = new QLabel(QCD_HUE_NAME, this); 
  label->setFixedSize(label->sizeHint());
  slidersLayout->addWidget(label, 3, 0, AlignVCenter|AlignRight);
  label = new QLabel(QCD_SATURATION_NAME, this); 
  label->setFixedSize(label->sizeHint());
  slidersLayout->addWidget(label, 4, 0, AlignVCenter|AlignRight);
  label = new QLabel(QCD_VALUE_NAME, this); 
  label->setFixedSize(label->sizeHint());
  slidersLayout->addWidget(label, 5, 0, AlignVCenter|AlignRight);
  /* END: HSV labels */
  /* BEGIN: HSV sliders */
  hSlider = new QSlider(0, 359, 10, 0, QSlider::Horizontal, this);
  hSlider->setFixedSize(hSlider->sizeHint());
  slidersLayout->addWidget(hSlider, 3, 1, AlignCenter);
  sSlider = new QSlider(0, 255, 10, 0, QSlider::Horizontal, this);
  sSlider->setFixedSize(sSlider->sizeHint());
  slidersLayout->addWidget(sSlider, 4, 1, AlignCenter);
  vSlider = new QSlider(0, 255, 10, 0, QSlider::Horizontal, this);
  vSlider->setFixedSize(vSlider->sizeHint());
  slidersLayout->addWidget(vSlider, 5, 1, AlignCenter);
  /* END: HSV sliders */
  /* BEGIN: HSV spinboxes */
  QSpinBox* hSpin = new QSpinBox(0, 359, 1, this);
  hSpin->setFixedSize(hSpin->sizeHint());
  slidersLayout->addWidget(hSpin, 3, 2, AlignCenter);
  QSpinBox* sSpin = new QSpinBox(0, 255, 1, this);
  sSpin->setFixedSize(sSpin->sizeHint());
  slidersLayout->addWidget(sSpin, 4, 2, AlignCenter);
  QSpinBox* vSpin = new QSpinBox(0, 255, 1, this);
  vSpin->setFixedSize(vSpin->sizeHint());
  slidersLayout->addWidget(vSpin, 5, 2, AlignCenter);
  /* END: HSV spinboxes */
  /* END: HSV */
  QHBoxLayout* okcancelLayout = new QHBoxLayout();
  mainLayout->addLayout(okcancelLayout,0);

  label = new QLabel(QCD_CHOSEN_NAME,this);
  label->setFixedSize(label->sizeHint());
  okcancelLayout->addWidget(label, 0);

  colorLabel = new QLabel(this); 
  colorLabel->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  colorLabel->setPalette(white);
  okcancelLayout->addWidget(colorLabel, 1);

  okcancelLayout->addStretch(1);

  QPushButton* ok = new QPushButton(QCD_OK_NAME, this);
  connect(ok, SIGNAL(clicked()), this, SLOT(accept()));
  okcancelLayout->addWidget(ok, 0);
  QPushButton* cancel = new QPushButton(QCD_CANCEL_NAME, this);
  connect(cancel, SIGNAL(clicked()), this, SLOT(reject()));
  okcancelLayout->addWidget(cancel, 0);
  cancel->setFixedSize(cancel->sizeHint());
  ok    ->setFixedSize(cancel->sizeHint());
  
  mainLayout->activate();
  resize(0, 0);
  mainLayout->freeze();

  /*
    This part is rather important as it connects all those above placed 
    widgets together. For each of the components we have the chain of the 
    connections as in following example:

    red slider -> red spin-box
    red slider <- red spin-box -> update the colorLabel
  */

  connect(rSlider, SIGNAL(valueChanged(int)), rSpin, SLOT(setValue(int)));
  connect(rSpin,  SIGNAL(valueChanged(int)), rSlider, SLOT(setValue(int)));
  connect(rSpin,  SIGNAL(valueChanged(int)), this, SLOT(updateFromRGB()));

  connect(gSlider, SIGNAL(valueChanged(int)), gSpin, SLOT(setValue(int)));
  connect(gSpin,  SIGNAL(valueChanged(int)), gSlider, SLOT(setValue(int)));
  connect(gSpin,  SIGNAL(valueChanged(int)), this, SLOT(updateFromRGB()));

  connect(bSlider, SIGNAL(valueChanged(int)), bSpin, SLOT(setValue(int)));
  connect(bSpin,  SIGNAL(valueChanged(int)), bSlider, SLOT(setValue(int)));
  connect(bSpin,  SIGNAL(valueChanged(int)), this, SLOT(updateFromRGB()));

  connect(hSlider, SIGNAL(valueChanged(int)), hSpin, SLOT(setValue(int)));
  connect(hSpin,  SIGNAL(valueChanged(int)), hsSelector, SLOT(setHue(int)));
  connect(hsSelector, SIGNAL(hueChanged(int)), hSlider, SLOT(setValue(int)));
  connect(hSpin,  SIGNAL(valueChanged(int)), this, SLOT(updateFromHSV()));
  connect(hSpin,  SIGNAL(valueChanged(int)), vSelector, SLOT(setHue(int)));

  connect(sSlider, SIGNAL(valueChanged(int)), sSpin, SLOT(setValue(int)));
  connect(sSpin,  SIGNAL(valueChanged(int)), hsSelector, SLOT(setSaturation(int)));
  connect(hsSelector, SIGNAL(saturationChanged(int)), sSlider, SLOT(setValue(int)));
  connect(sSpin,  SIGNAL(valueChanged(int)), this, SLOT(updateFromHSV()));
  connect(sSpin,  SIGNAL(valueChanged(int)), vSelector, SLOT(setSaturation(int)));

  connect(vSlider, SIGNAL(valueChanged(int)), vSpin, SLOT(setValue(int)));
  connect(vSpin,  SIGNAL(valueChanged(int)), vSelector, SLOT(setValue(int)));
  connect(vSelector, SIGNAL(valueChanged(int)), vSlider, SLOT(setValue(int)));
  connect(vSpin,  SIGNAL(valueChanged(int)), this, SLOT(updateFromHSV()));
}

/*! 
  The destructor in fact does nothing, the Qt library kills all the children of
  the dialog, and those are the only dynamically allocated things here.
*/
QColorDlg::~QColorDlg()
{
}

/*! 
  This function constructs a color selection dialog, runs it and returns the 
  resulting user selection (or initial selection if Cancel was pressed).

  @param initial initial color that will be shown in the dialog
  @return the color selected by user or initial color if Cancel was pressed
*/
QColor QColorDlg::getColor(const QColor& initial)
{
  QColorDlg dlg;

  dlg.setColor(initial);
  if (dlg.exec())
    return dlg.cur_color;
  else
    return initial;
}

/*! 
  This function sets the actual color in the dialog. It does it by updating 
  at first the values of RGB sliders and then calling updateFromRGB() function.

  @param c color to be set
*/
void QColorDlg::setColor(const QColor& c)
{
  rSlider->setValue(c.red());
  gSlider->setValue(c.green());
  bSlider->setValue(c.blue());
  updateFromRGB();
}

/*!
  This function recalculates the HSV values from the current RGB values and 
  updates the colorLabel to display appropriate color. As this function and 
  updateFromHSV() shouldn't overlap, this one first checks for #isHsvUpdating 
  and returns silently if true. In the other case it sets #isRgbUpdating so 
  that updateFromHSV() will not interfere.

  @see updateFromHSV()
*/
void QColorDlg::updateFromRGB()
{
  if (isHsvUpdating) return;
  isRgbUpdating = true;
  cur_color.setRgb(rSlider->value(), gSlider->value(), bSlider->value());
  colorLabel->setPalette(cur_color);

  int h,s,v;
  cur_color.hsv(&h,&s,&v);
  
  hSlider->setValue(h);
  sSlider->setValue(s);
  vSlider->setValue(v);
  isRgbUpdating = false;
}

/*!
  This function updates the RGB values from current HSV values. It is 
  complmentary to updateFromRGB().

  @see updateFromRGB()
*/
void QColorDlg::updateFromHSV()
{
  if (isRgbUpdating) return;
  isHsvUpdating = true;
  cur_color.setHsv(hSlider->value(), sSlider->value(), vSlider->value());
  colorLabel->setPalette(cur_color);

  rSlider->setValue(cur_color.red());
  gSlider->setValue(cur_color.green());
  bSlider->setValue(cur_color.blue());
  isHsvUpdating = false;
}

/*!
  This method constructs the HS selection widget with given \e parent and \e 
  name. The widget is always 180 pixels wide and 128 pixels high and it 
  contains the image with colors changing in Hue from 0 to 359 and in 
  Saturation from 0 to 255. In addition, the Value of the colors changes from 
  128 to 255 so that even low saturated colors don't blend into 
  indidtinguishable darkness.

  @param parent a widget's parent as passed to QLabel constructor
  @param name a widget's name as passed to QLabel constructor
*/
QHSSelector::QHSSelector(QWidget* parent, char * name) : QLabel(parent, name)
{
  QImage img(180,128,32);
  QColor c;
  for (int i = 0;i<180;i++)
    for (int j = 0;j<128;j++)
      {
	c.setHsv(2*i,2*j,128+j);
	img.setPixel(i,127-j,c.rgb());
      }
  QPixmap p;
  p.convertFromImage(img);
  setPixmap(p);
  setFixedSize(p.size());
  
  iHue = iSaturation = 0;
  setBackgroundMode(NoBackground);
}

/*!
  This destructor does nothing. The Qt framework takes care of all cleanup.
*/
QHSSelector::~QHSSelector()
{
}

/*!
  On repaint event, the default QLabel::paintEvent() is called. In this way the
  color chart image is redrawn. Later the diagonal cross is painted in the 
  position given by current #iHue, #iSaturation values.

  @param e QPaintEvent describing what should be repainted.
*/
void QHSSelector::paintEvent(QPaintEvent* e)
{
  QLabel::paintEvent(e);
  QPainter p(this);
  p.setPen(white);
  p.drawLine(iHue/2-5, 127-iSaturation/2-5, iHue/2+5, 127-iSaturation/2+5);
  p.drawLine(iHue/2+5, 127-iSaturation/2-5, iHue/2-5, 127-iSaturation/2+5);
}

/*!
  This function simply calls setHue() and setSaturation() with appropriate 
  values concerning the mouse position as described in mouse event.

  @param e QMouseEvent to be processed. 
*/
void QHSSelector::mouseMoveEvent(QMouseEvent* e)
{
  setHue(e->x()*2);
  setSaturation(256-e->y()*2);
}

/*!
  As the result of pressing mouse button within HSSelector should be the same 
  as dragging the mouse there, this function simply passes the event to 
  mouseMoveEvent().
  
  @param e QMouseEvent to be processed. 
*/
void QHSSelector::mousePressEvent(QMouseEvent* e)
{
  mouseMoveEvent(e);
}

/*!
  This function validates the value of the parameter h, sets #iHue to it and
  if the #iHue has changed emits signal hueChanged(). It also calls update of 
  the widget on the previous position of the cursor, so that the background is
  redrawn and a new cursor is painted appropriately.

  @param h hue value to be set
*/
void QHSSelector::setHue(int h)
{
  if (h < 0  ) h = 0;
  if (h > 359) h = 359;
  
  if (h != iHue)
    {
      update(iHue/2-6, 127-iSaturation/2-6, 12, 12);
      iHue = h;
      emit hueChanged(iHue);
    }
}

/*!
  This function validates the value of the parameter s, sets #iSaturation to 
  it. If the #iSaturation has changed emits signal saturationChanged(). It 
  also calls update of the widget on the previous position of the cursor, so 
  that the background is redrawn and a new cursor is painted appropriately.

  @param s saturation value to be set
*/
void QHSSelector::setSaturation(int s)
{
  if (s < 0  ) s = 0;
  if (s > 255) s = 255;

  if (s != iSaturation)
    {
      update(iHue/2-5, 127-iSaturation/2-5, 10, 10);
      iSaturation = s;
      emit saturationChanged(iSaturation);
    }
}

/*!
  This method constructs the QValueSelector object. It passes the arguments 
  \e parent and \e name to QLabel constructor. The widget that is constructed 
  has a fixed size of 10x128 pixels. The dafault background is a gradient from
  black to white.

  @param parent widget's parent passed to QLabel constructor
  @param name widget's name passed to QLabel constructor
*/
QValueSelector::QValueSelector(QWidget* parent, char * name) :
  QLabel(parent, name)
{
  QImage img(10,128,32);
  QColor c;

  iHue = iSaturation = iValue = 0;

  for (int j = 0;j<128;j++)
    {
      c.setHsv(iHue,iSaturation,255-2*j);
      for (int i = 0;i<10;i++)
	{
	  img.setPixel(i,j,c.rgb());
	}
    }
  QPixmap p;
  p.convertFromImage(img);
  setPixmap(p);
  setFixedSize(p.size());  
  setBackgroundMode(NoBackground);
}

/*! 
  The destructor does nothing; the Qt framework takes care of cleaning up.
*/
QValueSelector::~QValueSelector()
{
}

/*!
  This method processes paint event by calling the QLabel::paintEvent() so
  that the widget's background is updated and then drawing the slider in 
  appropriate position.

  @param e QPaintEvent object
*/
void QValueSelector::paintEvent(QPaintEvent * e)
{
  QLabel::paintEvent(e);
  QPainter p(this);
  p.setPen(white);
  p.drawLine(0,127-iValue/2,9,127-iValue/2);
  p.setPen(black);
  p.drawLine(0,127-iValue/2-1,9,127-iValue/2-1);
  p.drawLine(0,127-iValue/2+1,9,127-iValue/2+1);  
}
  
/*!
  This method simply calls setValue() function with appropriate value 
  calculated from the mouse y-coordinate.
*/
void QValueSelector::mouseMoveEvent(QMouseEvent* e)
{
  setValue(255 - e->y()*2);
}

/*!
  As the result of pressing the mouse button should be the same as dragging it,
  this method simply calls mouseMoveEvent() with the same argument.
*/
void QValueSelector::mousePressEvent(QMouseEvent* e)
{
  mouseMoveEvent(e);
}

/*!
  After validation of passed value \e v this method sets #iValue member to it. 
  If the #iValue has changed, appropriate updates are done on the widget's 
  background and the signal valueChanged() is emited.
*/
void QValueSelector::setValue(int v)
{
  if (v < 0  ) v = 0;
  if (v > 255) v = 255;
  if (v != iValue)
    {
      update(0,126-iValue/2,10,4);
      iValue = v;
      emit valueChanged(iValue);
    }
}

/*!
  This method updates the background image of the widget so that it contains 
  only colors with given hue and changing value. In the end it calls update()
  so that the whole widget is redrawn.

  @param h hue to be displayed on the slider.
*/
void QValueSelector::setHue(int h)
{
  static QImage img(10,128,32);
  QColor c;

  if (h != iHue)
    {
      iHue = h;
      for (int j = 0;j<128;j++)
	{
	  c.setHsv(iHue,iSaturation,255-2*j);
	  for (int i = 0;i<10;i++)
	    {
	      img.setPixel(i,j,c.rgb());
	    }
	}
      pixmap()->convertFromImage(img);
      update();
    }
}

/*!
  This method updates the background image of the widget so that it contains 
  only colors with given saturation and changing value. In the end it calls 
  update() so that the whole widget is redrawn.

  @param s saturation to be displayed on the slider.
*/
void QValueSelector::setSaturation(int s)
{
  static QImage img(10,128,32);
  QColor c;

  if (s != iSaturation)
    {
      iSaturation = s;
      for (int j = 0;j<128;j++)
	{
	  c.setHsv(iHue,iSaturation,255-2*j);
	  for (int i = 0;i<10;i++)
	    {
	      img.setPixel(i,j,c.rgb());
	    }
	}
      pixmap()->convertFromImage(img);
      update();
    }
}
