/* 
   This file is a part of Color Selection Dialog library.

   Copyright (C) 1999 ICe Machine
   Copyright (C) 1999 Jacek C. Wojdel <J.C.Wojdel@cs.tudelft.nl>
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*!
  \page index Qt Color Selection Dialog Documentation

  \author Jacek C. Wojdel 
    <a href="mailto:J.C.Wojdel@cs.tudelft.nl">J.C.Wojdel@cs.tudelft.nl</a>
  \version 1.0
  \date    7-06-1999

  This library gives you a new dialog class that can be used to let user chose
  a color from RGB or HSV color space.

  The usage of this library is explained in description of QColorDlg class 
  (which is the main class of this library) and shown as example in the main.cc
  file. In order to compile and run the example you have to edit the Makefile 
  that can be found in the distribution so that the paths to the Qt libraries
  and includes are correct.

  This library is licenced to you under the terms of \ref LICENCE, which can 
  be found in the LICENCE file.

  Qt is registered trademark of TrollTech 
  <a href="http://www.troll.no">http://www.troll.no</a>.

  <h3>Thanks:</h3>
  <ul>
  <li> <b>A. Wojdel</b> - for tracing down the bug that caused unclean 
  updates of the dialog in Win32 environment. </li>
  </ul>

  \bug I don't know any serius bugs yet.
  \bug There are however some limitations that could be resolved in future 
  releases (if there are any):
  <ul>
  <li> currently both QHSSelector and QValueSelector have fixed size an so are
  not appropriate for more generic use </li>
  <li> QValueSelector has fixed orientation, while maybe somebody would like
  to hsve it horizontal </li>
  <li> there is no nice way to incorporate new things in the dialog (like e.g.
  different color models, palette choosing etc.) </li>
  </ul>
*/
/*!
  \page LICENCE GNU Lesser General Public Licence

  \include <LICENCE>
*/


#ifndef QCOLORDLG_H
#define QCOLORDLG_H
/*!
  \file qcolordlg.h
  \brief Declaration of QColorDlg, QHSSelector and QValueSelector.

  This file contains the declarations of all the classes that are needed for
  color selection dialog. It should be included if you want to use any of 
  the following classes:
  <ul>
  <li> QColorDlg - main color selection dialog. </li>
  <li> QHSSelector - the Hue-Saturation chart embeded in QColorDlg. </li>
  <li> QValueSelector - the slider with Value-based gradient as beckground. </li>
  </ul>

  @see QColorDlg
*/

/*
  These define strings can be used to customize the labels used in the 
  color selection dialog. Some may decide to put here their national 
  equivalents, while some others can put here the calls to the localisation
  macros/functions.
*/
#define QCD_RED_NAME "Red"
#define QCD_GREEN_NAME "Green"
#define QCD_BLUE_NAME "Blue"
#define QCD_HUE_NAME "Hue"
#define QCD_SATURATION_NAME "Saturation"
#define QCD_VALUE_NAME "Value"
#define QCD_CHOSEN_NAME "Chosen color:"
#define QCD_OK_NAME "OK"
#define QCD_CANCEL_NAME "Cancel"

#include <qdialog.h>
#include <qcolor.h>
#include <qlabel.h>
#include <qrangecontrol.h>
class QSlider;

//! Color selection dialog class.
/*!
  This class can be used to allow user selection of the color using RGB or HSV
  color-model. For both of them there are both sliders and spinboxes for value
  selection. Additionally the HSV model is also represented in a well known 
  form of separate HS chart and V slider.

  This dialog may be used as any other in Qt, but it  provides also the 
  static function #getColor() that can be conviniently used as follows:
  \code
  #include "qcolordlg.h"

  ...

  QColor initial_color; // The color that dialog will start with
  QColor result;        // The resulting user's selection
  result = QColorDlg::getColor( initial_color );
  \endcode
  
*/
class QColorDlg : public QDialog
{
  Q_OBJECT
public:
  //! Constructor.
  QColorDlg(QWidget* parent = 0, const char * name = 0);
  //! Destructor.
  ~QColorDlg();

  //! Returns currently selected color.
  QColor color() const;

  //! Function that returns user's choice.
  static QColor getColor(const QColor& initial = black);

protected:
  //! Currently selected color.
  QColor cur_color;
  //! The label on which the color is displayed.
  QLabel* colorLabel;
  //! Red component slider.
  QSlider* rSlider;
  //! Green component slider.
  QSlider* gSlider;
  //! Blue component slider.
  QSlider* bSlider;
  //! Hue component slider.
  QSlider* hSlider;
  //! Saturation component slider.
  QSlider* sSlider;
  //! Value component slider.
  QSlider* vSlider;
  /*! 
    Flag for indicating that the dialog is within updating its state from 
    RGB controls.
  */
  bool isRgbUpdating;
  /*! 
    Flag for indicating that the dialog is within updating its state from 
    HSV controls.
  */  
  bool isHsvUpdating;
  
public slots:
  //! Sets current color in the dialog. 
  void setColor(const QColor& c);

protected slots:
  //! Updates the dialog from RGB controls.
  void updateFromRGB();
  //! Updates the dialog from HSV controls.
  void updateFromHSV();
  
};

//! Hue-Saturation chart selector class.
/*! 
  This class describes the Hue-Saturation chart as often found in the color 
  selection dialogs. In combination with QValueSelector it can be used to
  select a color from the HSV space.

  It behaves as any QRangeControl but with two degrees of freedom. Therefore 
  it contains the following signal/slot pairs:
  <ul>
  <li> <b>signal</b> hueChanged( int ) / <b>slot</b> setHue( int ) </li>
  <li> <b>signal</b> saturationChanged( int ) / 
       <b>slot</b> setSaturation( int ) </li>
  </ul>
  As usual, the signals are emited only if the appropriate value really 
  changed (either because of user action or after calling 'set' function).
  The ranges of Hue and Saturation components are predefined as:
  <ul>
  <li> Hue - <0 ... 359> </li>
  <li> Saturation - <0 ... 255>  </li>
  </ul>
  
  <b>NOTE:</b> Despite behaviour similarities, this class derives only from 
  QLabel, not from QRangeControl.

  @see QValueSelector
*/
class QHSSelector : public QLabel
{
  Q_OBJECT
public:
  //! Constructor.
  QHSSelector(QWidget* parent = 0, char * name = 0);
  //! Destructor.
  ~QHSSelector();

public:
  //! Returns the hue value.
  int hue() const; 
  //! Returns the saturation value.
  int saturation() const;

public slots:
  //! Set the hue value of selector.
  void setHue(int h);
  //! Set the saturation value of selector.
  void setSaturation(int s);

signals:
  //! Emited when hue changed.
  void hueChanged(int);
  //! Emited when saturation changed.
  void saturationChanged(int);

protected:
  //! Current hue value.
  int iHue;
  //! Current saturation value.
  int iSaturation;
  //! Repaints the HS chart and draws the selection cross.
  virtual void paintEvent(QPaintEvent * e);
  
  //! Used for user interaction.
  virtual void mouseMoveEvent(QMouseEvent* e);
  //! Used for user interaction.
  virtual void mousePressEvent(QMouseEvent* e);
};

//! Slider with Value-based gradient background
/*!
  This class provides a slider with the background changing from 0 to maximum
  Value of the specified color. It can be used together with QHSSelector to 
  allow selection of a color in HSV color space.
  
  It contains only one signal, as there is only one value that can be changed
  by user. Yet there are three 'set' signals which is neccessary as the 
  control also needs information about Hue and Saturation to properly update 
  its background. In the code it could be connected to the QHSSelector widget
  in the following manner:
  \code
  QHSSelector* hsWidget;
  QValueSelector* vWidget;

  ... (set up the widgets appropriately within their parent) ...

  connect( hsWidget, SIGNAL( hueChanged( int ) ), vWidget, SLOT( setHue( int ) ) );
  connect( hsWidget, SIGNAL( saturationChanged( int ) ), vWidget, SLOT( setSaturation( int ) ) );

  ...

  QColor current_color = QColor( hsWidget->hue(), hsWidget->saturation(), vWidget->value() ,QColor::Hsv );
  \endcode

  The Value can be in a range <0 ... 255>.
  @see QHSSelector
*/
class QValueSelector : public QLabel
{
  Q_OBJECT
public:
  //! Constructor.
  QValueSelector(QWidget* parent = 0, char * name = 0);
  //! Destructor.
  ~QValueSelector();

  //! Returns current value.
  int value() const;

public slots:
  //! Sets the surrent Value.
  void setValue(int v);
  //! Sets the surrent Hue.
  void setHue(int h);
  //! Sets the surrent Saturation.
  void setSaturation(int s);

signals:
  //! Emited when value changed.
  void valueChanged(int);
  
protected:
  //! Current Hue.
  int iHue;
  //! Current Saturation.
  int iSaturation;
  //! Current Value.
  int iValue;

  //! Repaints gradient changing background together with the slider mark.
  virtual void paintEvent(QPaintEvent * e);
  
  //! Used for user interaction.
  virtual void mouseMoveEvent(QMouseEvent* e);
  //! Used for user interaction.
  virtual void mousePressEvent(QMouseEvent* e);
};

inline QColor QColorDlg::color() const
{
  return cur_color;
}

inline int QHSSelector::hue() const
{
  return iHue;
}

inline int QHSSelector::saturation() const
{
  return iSaturation;
}

inline int QValueSelector::value() const
{
  return iValue;
}

#endif
